commandCodes = {
    0x01: "CONNECT",
    0x1B: "GET_CCP_VERSION",
    0x17: "EXCHANGE_ID",
    0x12: "GET_SEED",
    0x13: "UNLOCK",
    0x02: "SET_MTA",
    0x03: "DNLOAD",
    0x23: "DNLOAD_6",
    0x04: "UPLOAD",
    0x0F: "SHORT_UP",
    0x11: "SELECT_CAL_PAGE",
    0x14: "GET_DAQ_SIZE",
    0x15: "SET_DAQ_POINTER",
    0x16: "WRITE_DAQ",
    0x06: "START_STOP",
    0x07: "DISCONNECT",
    0x0C: "SET_S_STATUS",
    0x0D: "GET_S_STATUS",
    0x0E: "BUILD_CHKSUM",
    0x10: "CLEAR_MEMORY",
    0x18: "PROGRAM",
    0x22: "PROGRAM_6",
    0x19: "MOVE",
    0x05: "TEST",
    0x09: "GET_ACTIVE_CAL_PAGE",
    0x08: "START_STOP_ALL",
    0x20: "DIAG_SERVICE",
    0x21: "ACTION_SERVICE"
}
